import { createVNode as _createVNode } from "vue";
// Styles
import "./VList.css"; // Components

import { VListChildren } from "./VListChildren.mjs"; // Composables

import { makeBorderProps, useBorder } from "../../composables/border.mjs";
import { makeDensityProps, useDensity } from "../../composables/density.mjs";
import { makeDimensionProps, useDimension } from "../../composables/dimensions.mjs";
import { makeElevationProps, useElevation } from "../../composables/elevation.mjs";
import { makeRoundedProps, useRounded } from "../../composables/rounded.mjs";
import { makeTagProps } from "../../composables/tag.mjs";
import { useBackgroundColor } from "../../composables/color.mjs";
import { makeThemeProps, provideTheme } from "../../composables/theme.mjs";
import { makeNestedProps, useNested } from "../../composables/nested/nested.mjs";
import { createList } from "./list.mjs"; // Utilities

import { computed, toRef } from 'vue';
import { genericComponent, useRender } from "../../util/index.mjs"; // Types

const parseItems = items => {
  if (!items) return undefined;
  return items.map(item => {
    if (typeof item === 'string') return {
      type: 'item',
      value: item,
      title: item
    };
    const {
      $type,
      $children,
      ...props
    } = item;
    if ($type === 'subheader') return {
      type: 'subheader',
      props
    };
    if ($type === 'divider') return {
      type: 'divider',
      props
    };
    return {
      type: 'item',
      props,
      children: parseItems($children)
    };
  });
};

export const VList = genericComponent()({
  name: 'VList',
  props: {
    color: String,
    disabled: Boolean,
    lines: {
      type: String,
      default: 'one'
    },
    nav: Boolean,
    items: Array,
    ...makeNestedProps({
      selectStrategy: 'single-leaf',
      openStrategy: 'multiple'
    }),
    ...makeBorderProps(),
    ...makeDensityProps(),
    ...makeDimensionProps(),
    ...makeElevationProps(),
    ...makeRoundedProps(),
    ...makeTagProps(),
    ...makeThemeProps()
  },
  emits: {
    'update:selected': val => true,
    'update:opened': val => true,
    'click:open': value => true,
    'click:select': value => true
  },

  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const items = computed(() => parseItems(props.items));
    const {
      themeClasses
    } = provideTheme(props);
    const {
      backgroundColorClasses,
      backgroundColorStyles
    } = useBackgroundColor(toRef(props, 'color'));
    const {
      borderClasses
    } = useBorder(props);
    const {
      densityClasses
    } = useDensity(props);
    const {
      dimensionStyles
    } = useDimension(props);
    const {
      elevationClasses
    } = useElevation(props);
    const {
      roundedClasses
    } = useRounded(props);
    const {
      open,
      select
    } = useNested(props);
    createList();
    useRender(() => {
      return _createVNode(props.tag, {
        "class": ['v-list', {
          'v-list--disabled': props.disabled,
          'v-list--nav': props.nav,
          [`v-list--${props.lines}-line`]: true
        }, themeClasses.value, backgroundColorClasses.value, borderClasses.value, densityClasses.value, elevationClasses.value, roundedClasses.value],
        "style": [backgroundColorStyles.value, dimensionStyles.value]
      }, {
        default: () => [_createVNode(VListChildren, {
          "items": items.value
        }, {
          default: slots.default,
          item: slots.item,
          title: slots.title,
          subtitle: slots.subtitle,
          header: slots.header
        })]
      });
    });
    return {
      open,
      select
    };
  }

});
//# sourceMappingURL=VList.mjs.map