import { createTextVNode as _createTextVNode, createVNode as _createVNode, Fragment as _Fragment } from "vue";
// Styles
import "./VSelect.css"; // Components

import { VChip } from "../VChip/index.mjs";
import { VDefaultsProvider } from "../VDefaultsProvider/index.mjs";
import { VList, VListItem } from "../VList/index.mjs";
import { VMenu } from "../VMenu/index.mjs";
import { VTextField } from "../VTextField/index.mjs"; // Composables

import { makeTransitionProps } from "../../composables/transition.mjs";
import { useForwardRef } from "../../composables/forwardRef.mjs";
import { useLocale } from "../../composables/locale.mjs";
import { useProxiedModel } from "../../composables/proxiedModel.mjs"; // Utility

import { computed, ref, watch } from 'vue';
import { genericComponent, propsFactory, useRender, wrapInArray } from "../../util/index.mjs"; // Types

export function genItem(item) {
  var _ref;

  return {
    title: String((_ref = typeof item === 'object' ? item.title : item) != null ? _ref : ''),
    value: typeof item === 'object' ? item.value : item
  };
}
export const makeSelectProps = propsFactory({
  chips: Boolean,
  closableChips: Boolean,
  eager: Boolean,
  hideNoData: Boolean,
  hideSelected: Boolean,
  items: {
    type: Array,
    default: () => []
  },
  menuIcon: {
    type: String,
    default: '$dropdown'
  },
  modelValue: {
    type: [Number, String, Array],
    default: () => []
  },
  multiple: Boolean,
  noDataText: {
    type: String,
    default: '$vuetify.noDataText'
  },
  openOnClear: Boolean
}, 'select');
export const VSelect = genericComponent()({
  name: 'VSelect',
  props: { ...makeSelectProps(),
    ...makeTransitionProps({
      transition: 'scale-transition'
    })
  },
  emits: {
    'update:modelValue': val => true
  },

  setup(props, _ref2) {
    let {
      slots
    } = _ref2;
    const {
      t
    } = useLocale();
    const vTextFieldRef = ref();
    const activator = ref();
    const menu = ref(false);
    const items = computed(() => props.items.map(genItem));
    const model = useProxiedModel(props, 'modelValue', [], v => wrapInArray(v), v => props.multiple ? v : v[0]);
    const selections = computed(() => {
      const array = [];
      let index = 0;

      for (const unwrapped of model.value) {
        const item = genItem(unwrapped);
        const found = array.find(selection => selection.value === item.value);

        if (found == null) {
          array.push({ ...item,
            index
          });
          index++;
        }
      }

      return array;
    });
    const selected = computed(() => selections.value.map(selection => selection.value));

    function onClear(e) {
      model.value = [];

      if (props.openOnClear) {
        menu.value = true;
      }
    }

    function onClickControl() {
      if (props.hideNoData && !items.value.length) return;
      menu.value = true;
    }

    function onKeydown(e) {
      if (['Enter', 'ArrowDown', ' '].includes(e.key)) {
        menu.value = true;
      }

      if (['Escape', 'Tab'].includes(e.key)) {
        menu.value = false;
      }
    }

    function select(item) {
      if (props.multiple) {
        const index = selections.value.findIndex(selection => selection.value === item.value);

        if (index === -1) {
          model.value.push(item.value);
        } else {
          model.value = selected.value.filter(selection => selection !== item.value);
        }
      } else {
        model.value = [item.value];
        menu.value = false;
      }
    }

    watch(() => vTextFieldRef.value, val => {
      activator.value = val.$el.querySelector('.v-input__control');
    });
    useRender(() => {
      const hasChips = !!(props.chips || slots.chip);
      return _createVNode(VTextField, {
        "ref": vTextFieldRef,
        "class": ['v-select', {
          'v-select--active-menu': menu.value,
          'v-select--chips': !!props.chips,
          [`v-select--${props.multiple ? 'multiple' : 'single'}`]: true
        }],
        "appendInnerIcon": props.menuIcon,
        "readonly": true,
        "onClick:clear": onClear,
        "onClick:input": onClickControl,
        "onClick:control": onClickControl,
        "onBlur": () => menu.value = false,
        "modelValue": model.value.join(', '),
        "onKeydown": onKeydown
      }, { ...slots,
        default: () => _createVNode(_Fragment, null, [activator.value && _createVNode(VMenu, {
          "modelValue": menu.value,
          "onUpdate:modelValue": $event => menu.value = $event,
          "activator": activator.value,
          "contentClass": "v-select__content",
          "eager": props.eager,
          "openOnClick": false,
          "transition": props.transition
        }, {
          default: () => [_createVNode(VList, {
            "selected": selected.value,
            "selectStrategy": props.multiple ? 'independent' : 'single-independent'
          }, {
            default: () => [!items.value.length && !props.hideNoData && _createVNode(VListItem, {
              "title": t(props.noDataText)
            }, null), items.value.map(item => _createVNode(VListItem, {
              "title": item.title,
              "value": item.value,
              "onMousedown": e => e.preventDefault(),
              "onClick": () => select(item)
            }, null))]
          })]
        }), selections.value.map((selection, index) => {
          function onChipClose(e) {
            e.stopPropagation();
            e.preventDefault();
            select(selection);
          }

          const slotProps = {
            'onClick:close': onChipClose,
            modelValue: true
          };
          return _createVNode("div", {
            "class": "v-select__selection"
          }, [hasChips && _createVNode(VDefaultsProvider, {
            "defaults": {
              VChip: {
                closable: props.closableChips,
                size: 'small',
                text: selection.title
              }
            }
          }, {
            default: () => [slots.chip ? slots.chip({
              props: slotProps,
              selection
            }) : _createVNode(VChip, slotProps, null)]
          }), !hasChips && (slots.selection ? slots.selection({
            selection
          }) : _createVNode("span", {
            "class": "v-select__selection-text"
          }, [selection.title, props.multiple && index < selections.value.length - 1 && _createVNode("span", {
            "class": "v-select__selection-comma"
          }, [_createTextVNode(",")])]))]);
        })])
      });
    });
    return useForwardRef({}, vTextFieldRef);
  }

});
//# sourceMappingURL=VSelect.mjs.map